import structlog
import logging
import sys

from structlog.processors import JSONRenderer
from structlog.stdlib import filter_by_level
from structlog.stdlib import add_log_level_number


def rename_message_key(_, __, event_dict):
    event_dict["message"] = event_dict["event"]
    event_dict.pop("event", None)
    return event_dict


def increase_level_numbers(_, __, event_dict):
    event_dict["level"] = event_dict["level_number"] * 10
    event_dict.pop("level_number", None)
    return event_dict


def get_logger(name=__name__):

    logging.basicConfig(
        format="%(message)s",
        stream=sys.stdout,
        level=logging.INFO,
    )

    processors = [
        filter_by_level,
        rename_message_key,
        add_log_level_number,
        increase_level_numbers,
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
        structlog.processors.UnicodeDecoder(),
        JSONRenderer(),
    ]
    structlog.configure(
        context_class=dict,
        logger_factory=structlog.stdlib.LoggerFactory(),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
        processors=processors,
    )
    return structlog.get_logger(name).new()



try:
    import ddtrace
    from ddtrace.helpers import get_correlation_ids

    ddtrace_available = True
except ImportError:
    ddtrace_available = False


def is_ddtrace_available():
    return ddtrace_available


def tracer_injection(logger, log_method, event_dict):
    # get correlation ids from current tracer context
    trace_id, span_id = get_correlation_ids()

    # Add ids to Structlog event dictionary
    event_dict["dd.trace_id"] = trace_id or 0
    event_dict["dd.span_id"] = span_id or 0

    # add the env, service, and version configured for the tracer
    event_dict["dd.env"] = ddtrace.config.env or ""
    event_dict["dd.service"] = ddtrace.config.service or ""
    event_dict["dd.version"] = ddtrace.config.version or ""

    return event_dict


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="python-bufflog",
    version="0.1.2",
    author="David Gasquez",
    author_email="davidgasquez@buffer.com",
    description="Python logger for Buffer services",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bufferapp/python-bufflog",
    install_requires=["structlog"],
    packages=setuptools.find_packages(),
)
